@@ -184,6 +184,8 @@ static void destroy_tls_state(ErlNifEnv *env, void *data) {
     }
 }
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined LIBRESSL_VERSION_NUMBER
+
 static void locking_callback(int mode, int n, const char *file, int line) {
     if (mode & CRYPTO_LOCK)
         enif_mutex_lock(mtx_buf[n]);
@@ -199,6 +201,8 @@ static int atomic_add_callback(int *pointer, int amount, int type, const char *f
     return __sync_add_and_fetch(pointer, amount);
 }
 
+#endif
+
 static int load(ErlNifEnv *env, void **priv, ERL_NIF_TERM load_info) {
     int i;
 
@@ -466,14 +470,19 @@ static int ssl_sni_callback(const SSL *s, int *foo, void *data) {
 static ERL_NIF_TERM ssl_error(ErlNifEnv *env, const char *errstr) {
     size_t rlen;
     ErlNifBinary err;
+    char error_string[256];
+    size_t error_string_length;
     size_t errstrlen = strlen(errstr);
     unsigned long error_code = ERR_get_error();
-    char *error_string = error_code ? ERR_error_string(error_code, NULL) : NULL;
-    size_t error_string_length = error_string ? strlen(error_string) : 0;
-    if (error_code)
+
+    if (error_code) {
+        ERR_error_string_n(error_code, error_string, sizeof(error_string));
+        error_string_length = strlen(error_string),
         rlen = errstrlen + error_string_length + 2;
-    else
+    } else {
+        error_string_length = 0;
         rlen = errstrlen;
+    }
     enif_alloc_binary(rlen, &err);
     memcpy(err.data, errstr, errstrlen);
     if (error_code) {
